#include "InteriorBoundary.h"
#include "FiniteElemSpace.h"
#include "BasisFunction.h"
#include "Serializer.h"

namespace AMDiS {

  void BoundaryObject::setReverseMode(BoundaryObject &otherBound, 
				      FiniteElemSpace *feSpace)
  {
    FUNCNAME("BoundaryObject::setReverseMode()");

    bool otherMode = false;

    const BasisFunction *basFcts = feSpace->getBasisFcts();
    int nBasFcts = basFcts->getNumber();
    std::vector<DegreeOfFreedom> localDofs0(nBasFcts), localDofs1(nBasFcts);
    
    switch (feSpace->getMesh()->getDim()) {
    case 2:
      ERROR_EXIT("Not yet implemented!\n");
      break;
    case 3:
      if (ithObj == 2 || ithObj == 3) {
	basFcts->getLocalIndices(el, feSpace->getAdmin(), localDofs0);
	basFcts->getLocalIndices(otherBound.el, feSpace->getAdmin(), localDofs1);
	otherMode = (localDofs0[0] != localDofs1[0]);
      }

      break;
    default:
      ERROR_EXIT("This should not happen!\n");
    }

    otherBound.reverseMode = otherMode;
  }


  AtomicBoundary& InteriorBoundary::getNewAtomic(int rank)
  {
    boundary[rank].resize(boundary[rank].size() + 1);
    return boundary[rank][boundary[rank].size() - 1];
  }


  void InteriorBoundary::serialize(std::ostream &out)
  {
    int mSize = boundary.size();
    SerUtil::serialize(out, mSize);
    for (RankToBoundMap::iterator it = boundary.begin(); it != boundary.end(); ++it) {
      int rank = it->first;
      int boundSize = it->second.size();
      SerUtil::serialize(out, rank);
      SerUtil::serialize(out, boundSize);
      for (int i = 0; i < boundSize; i++) {
	AtomicBoundary &bound = (it->second)[i];

	SerUtil::serialize(out, bound.rankObj.elIndex);
	SerUtil::serialize(out, bound.rankObj.elType);
	SerUtil::serialize(out, bound.rankObj.subObj);
	SerUtil::serialize(out, bound.rankObj.ithObj);
	SerUtil::serialize(out, bound.rankObj.reverseMode);

	SerUtil::serialize(out, bound.neighObj.elIndex);
	SerUtil::serialize(out, bound.neighObj.elType);
	SerUtil::serialize(out, bound.neighObj.subObj);
	SerUtil::serialize(out, bound.neighObj.ithObj);
	SerUtil::serialize(out, bound.neighObj.reverseMode);
      }
    }
  }


  void InteriorBoundary::deserialize(std::istream &in, 
				     std::map<int, Element*> &elIndexMap)
  {
    int mSize = 0;
    SerUtil::deserialize(in, mSize);
    for (int i = 0; i < mSize; i++) {
      int rank = 0;
      int boundSize = 0;
      SerUtil::deserialize(in, rank);
      SerUtil::deserialize(in, boundSize);

      boundary[rank].resize(boundSize);
      for (int i = 0; i < boundSize; i++) {
	AtomicBoundary &bound = boundary[rank][i];

	SerUtil::deserialize(in, bound.rankObj.elIndex);
	SerUtil::deserialize(in, bound.rankObj.elType);
	SerUtil::deserialize(in, bound.rankObj.subObj);
	SerUtil::deserialize(in, bound.rankObj.ithObj);
	SerUtil::deserialize(in, bound.rankObj.reverseMode);

	SerUtil::deserialize(in, bound.neighObj.elIndex);
	SerUtil::deserialize(in, bound.neighObj.elType);
	SerUtil::deserialize(in, bound.neighObj.subObj);
	SerUtil::deserialize(in, bound.neighObj.ithObj);
	SerUtil::deserialize(in, bound.neighObj.reverseMode);

	bound.rankObj.el = elIndexMap[bound.rankObj.elIndex];
	bound.neighObj.el = NULL;
      }
    }
  }
}
